package com.kddi.helpers.cli;

import java.text.SimpleDateFormat;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.ListObjectsRequest;
import com.amazonaws.services.s3.model.ListVersionsRequest;
import com.amazonaws.services.s3.model.ObjectListing;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import com.amazonaws.services.s3.model.S3VersionSummary;
import com.amazonaws.services.s3.model.VersionListing;
import com.kddi.helpers.provider.ClientProvider;

/**
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.helpers.cli.DeleteObjects -Dexec.args="-b bucket -p prefix"
 * </pre>
 * 
 * @author ito
 *
 */
public class DeleteObjects
{
  private static final Logger log = LoggerFactory.getLogger(DeleteObjects.class);
  private final String NULL_VERSION = "null";
  private final String NULL_PADDING = "                                ";
  public static final Boolean WITH_VERSIONS = true;
  public static final Boolean OBJECT_ONLY = false;

  public static void main(String[] args)
  {
    String b, p;

    try
    {
      Options opt = new Options();
      opt.addOption("h", false, "Print help for this application");
      opt.addOption("b", true, "bucket name");
      opt.addOption("p", true, "key prefix");

      DefaultParser parser = new DefaultParser();
      CommandLine cl = parser.parse(opt, args);

      if (cl.hasOption('h'))
      {
        HelpFormatter f = new HelpFormatter();
        f.printHelp("OptionsTip", opt);
        System.exit(0);
      }

      if (cl.hasOption("b"))
      {
        b = cl.getOptionValue("b");
      }
      else
      {
        b = ClientProvider.getInstance().bucketName1;
        log.debug("using default bucketName : {}", b);
      }

      if (cl.hasOption("p"))
      {
        p = cl.getOptionValue("p");
      }
      else
      {
        p = "";
        log.debug("using default prefix : '{}'", b);
      }

      new DeleteObjects().exec(b, p, DeleteObjects.OBJECT_ONLY);
    }
    catch (ParseException e)
    {
      log.error("HELPER: Error Occurs: ", e);
    }
  }

  /**
   * オブジェクトとバージョンを削除する
   * 
   * @param bucketName
   * @param prefix keyname prefix
   */
  public void exec(String bucketName, String prefix, Boolean isDeleteVersions)
  {
    log.debug("HELPER: deleteObjectsVersions - [L]atest, [D]eleted");

    try
    {
      if (isDeleteVersions)
      {
        deleteVersions(bucketName, prefix);
      }
      else
      {
        deleteObjects(bucketName, prefix);
      }
    }
    catch (AmazonS3Exception ase)
    {
      log.warn("HELPER: ", ase);
    }
  }

  private void deleteObjects(String bucketName, String prefix)
  {
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();

    ListObjectsRequest listObjectsRequest = new ListObjectsRequest().withBucketName(bucketName).withPrefix(prefix);
    ObjectListing olist;
    do
    {
      olist = s3Client.listObjects(listObjectsRequest);
      log.debug("object marker [{}]", olist.getMarker());
      if (olist.getObjectSummaries().size() == 0)
      {
        log.debug("object list empty.");
      }

      for (S3ObjectSummary o : olist.getObjectSummaries())
      {
        s3Client.deleteObject(bucketName, o.getKey());
        log.debug("{} - {} deleted.", bucketName, o.getKey());
      }
      listObjectsRequest.setMarker(olist.getNextMarker());
    }
    while (olist.isTruncated());
  }

  private void deleteVersions(String bucketName, String prefix)
  {
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String ver_done = "deleted       - {}@[{}] {} {} {} {} {}byte";
    String ver_fail = "delete failed - {}@[{}] {} {} {} {} {}byte";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS zzz ZZZZZ");

    String latest = null;
    String deleted = null;

    ListVersionsRequest listVersionsRequest = new ListVersionsRequest().withBucketName(bucketName).withPrefix(prefix);
    VersionListing vlist;
    do
    {
      latest = "   ";
      deleted = "   ";

      vlist = s3Client.listVersions(listVersionsRequest);
      log.debug("version marker [{}]", vlist.getVersionIdMarker());
      if (vlist.getVersionSummaries().size() == 0)
      {
        log.debug("version list empty.");
      }

      for (S3VersionSummary v : vlist.getVersionSummaries())
      {
        String vid = v.getVersionId();
        if (NULL_VERSION.equals(vid))
        {
          vid = NULL_PADDING + vid;
        }

        try
        {
          if (v.isLatest())
          {
            latest = "[L]";
          }

          if (v.isDeleteMarker())
          {
            deleted = "[D]";
          }

          s3Client.deleteVersion(bucketName, v.getKey(), v.getVersionId());
          log.debug("HELPER: " + ver_done, v.getBucketName(), sdf.format(v.getLastModified()), latest, deleted, vid, v.getKey(), v.getSize());
        }
        catch (AmazonS3Exception ase)
        {
          if (v.isLatest())
          {
            latest = "[L]";
          }

          if (v.isDeleteMarker())
          {
            deleted = "[D]";
          }

          log.warn("HELPER: " + ver_fail, v.getBucketName(), sdf.format(v.getLastModified()), latest, deleted, vid, v.getKey(), v.getSize());
          log.warn("HELPER: ", ase);
        }
      }
      listVersionsRequest.setKeyMarker(vlist.getNextVersionIdMarker());
    }
    while (vlist.isTruncated());
  }
}
