package com.kddi.helpers.cli;

import java.text.SimpleDateFormat;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.ListVersionsRequest;
import com.amazonaws.services.s3.model.S3VersionSummary;
import com.amazonaws.services.s3.model.VersionListing;
import com.kddi.helpers.provider.ClientProvider;

/**
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.helpers.cli.ShowObjectsVersions -Dexec.args="-b bucket -p prefix [-d]"
 * </pre>
 * 
 * @author ito
 *
 */
public class ShowObjectsVersions
{
  private static final Logger log = LoggerFactory.getLogger(ShowObjectsVersions.class);

  public static void main(String[] args)
  {
    String b, p;

    try
    {
      Options opt = new Options();
      opt.addOption("h", false, "Print help for this application");
      opt.addOption("b", true, "bucket name");
      opt.addOption("p", true, "key prefix");
      opt.addOption("d", false, "show deleted marker key");

      DefaultParser parser = new DefaultParser();
      CommandLine cl = parser.parse(opt, args);

      if (cl.hasOption('h'))
      {
        HelpFormatter f = new HelpFormatter();
        f.printHelp("OptionsTip", opt);
        System.exit(0);
      }

      if (cl.hasOption("b"))
      {
        b = cl.getOptionValue("b");
      }
      else
      {
        b = ClientProvider.getInstance().bucketName1;
        log.debug("using default bucketName : {}", b);
      }

      if (cl.hasOption("p"))
      {
        p = cl.getOptionValue("p");
      }
      else
      {
        p = "";
        log.debug("using default prefix : '{}'", b);
      }
      Boolean isShowDeletedKey = false;
      if (cl.hasOption("d"))
      {
        isShowDeletedKey = true;
      }
      new ShowObjectsVersions().exec(b, p, isShowDeletedKey);
    }
    catch (ParseException e)
    {
      log.error("HELPER: Error Occurs:", e);
    }
  }

  /**
   * オブジェクトとバージョンの一覧を出力する
   * 
   * @param bucketName
   * @param prefix
   */
  public void exec(String bucketName, String prefix, Boolean isShowDeletedKey)
  {
    if (bucketName == null || prefix == null || isShowDeletedKey == null)
    {
      throw new IllegalArgumentException();
    }
    log.debug("HELPER: showObjectsVersions - [L]atest, [D]eleted");

    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String ver = "{}@[{}] {} {} {} {} {}byte";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS zzz ZZZZZ");
    ListVersionsRequest listVersionsRequest = new ListVersionsRequest().withBucketName(bucketName).withPrefix(prefix);
    VersionListing vlist;
    do
    {
      vlist = s3Client.listVersions(listVersionsRequest);
      log.debug("version marker [{}]", vlist.getVersionIdMarker());
      if (vlist.getVersionSummaries().size() == 0)
      {
        log.debug("version list empty.");
      }

      for (S3VersionSummary v : vlist.getVersionSummaries())
      {
        String vid = v.getVersionId();
        if (vid.length() < 5)
        {
          StringBuilder sb = new StringBuilder();
          for (int i = 0; i < 32; i++)
          {
            sb.append(" ");
          }
          vid = sb.toString() + vid;
        }
        String latest = "   ";
        if (v.isLatest())
        {
          latest = "[L]";
        }

        String delete = "   ";
        if (v.isDeleteMarker())
        {
          delete = "[D]";
        }

        log.debug("HELPER: " + ver, v.getBucketName(), sdf.format(v.getLastModified()), latest, delete, vid, v.getKey(), v.getSize());
      }
      listVersionsRequest.setKeyMarker(vlist.getVersionIdMarker());
    }
    while (vlist.isTruncated());
  }
}
