package com.kddi.helpers.creator;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.event.ProgressEvent;
import com.amazonaws.event.ProgressEventType;
import com.amazonaws.event.ProgressListener;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.PutObjectRequest;
import com.amazonaws.services.s3.transfer.Copy;
import com.amazonaws.services.s3.transfer.Download;
import com.amazonaws.services.s3.transfer.TransferManager;
import com.amazonaws.services.s3.transfer.Upload;

public class MockCreator
{
  private static final Logger log = LoggerFactory.getLogger(MockCreator.class);
  private AmazonS3 client;
  public static final long SIZE_GB = 1024L * 1024L * 1024L;
  public static final long SIZE_MB = 1024L * 1024L;
  private File tmpFile;
  public String filePath;

  public MockCreator(AmazonS3 client)
  {
    log.debug("HELPER: set AmazonS3 client.");
    this.client = client;
  }

  /**
   * "0123456789"を繰り返し書き込んだテキストファイルをPROJECT_ROOT/tmp以下に生成し、objectを保持する。
   * 
   * @param fileName String ファイル名
   * @param contentSize long ファイルサイズ
   * @return MockCreator
   */
  public MockCreator createTextFile(String fileName, long contentSize)
  {
    try
    {
      String[] paths = { "tmp", fileName };
      tmpFile = new File(String.join(File.separator, paths));
      if (tmpFile.exists())
      {
        if (tmpFile.length() > contentSize)
        {
          tmpFile.delete();
          tmpFile.createNewFile();
        }
      }
      else
      {
        tmpFile.createNewFile();
      }

      // 1KB strings
      int buffer_size = (int) (16 * 1024L);
      FileWriter wr = new FileWriter(tmpFile, true);
      BufferedWriter bw = new BufferedWriter(wr, buffer_size);
      StringBuilder sb = new StringBuilder();
      int in = 0;
      for (int i = 0; i < 1024L; i++)
      {
        if (in > 9)
        {
          in = 0;
        }
        sb.append(in);
        in++;
      }

      log.debug("HELPER: {} creating..", tmpFile.getCanonicalPath());

      while (tmpFile.length() <= contentSize - buffer_size)
      {
        bw.write(sb.toString());
      }
      bw.close();

      long remain = contentSize - tmpFile.length();
      in = 0;
      if (remain > 0)
      {
        wr = new FileWriter(tmpFile, true);
      }
      for (int i = 0; i < remain; i++)
      {
        if (in > 9)
        {
          in = 0;
        }
        wr.write(in);
        in++;
      }
      wr.close();

      filePath = tmpFile.getAbsolutePath();
      log.debug("HELPER: created {} {}byte", filePath, tmpFile.length());
    }
    catch (IOException ioe)
    {
      log.error("HELPER: Error Occurs:", ioe);
    }
    return this;
  }

  public void duplicate(String bucketName, int amount) throws FileNotFoundException
  {
    this.uploadTo(bucketName);

    TransferManager tm = new TransferManager(this.client);
    for (int i = 0; i < amount; i++)
    {
      String dest_keyName = tmpFile.getName() + ".0" + i;
      tm.copy(bucketName, tmpFile.getName(), bucketName, dest_keyName);
      log.debug("HELPER: <{}>{} duplicated.", bucketName, dest_keyName);
    }
  }

  /**
   * 指定bucketに保持しているobjectをアップロードする。
   * 
   * @param bucketName String
   * @throws FileNotFoundException 保持している筈のobjectが存在しない時に発生する
   */
  public void uploadTo(String bucketName) throws FileNotFoundException
  {
    if (tmpFile == null || tmpFile.length() < 1)
    {
      throw new FileNotFoundException("upload before create file !");
    }

    Upload upload;
    ExecutorService execsv = Executors.newFixedThreadPool(15);
    TransferManager tm = new TransferManager(this.client, execsv);
    ScheduledExecutorService ex = Executors.newSingleThreadScheduledExecutor();

    try
    {
      PutObjectRequest request = new PutObjectRequest(bucketName, tmpFile.getName(), tmpFile);
      log.debug("HELPER: {} uploading to {}..", tmpFile.getName(), bucketName);
      upload = tm.upload(request);

      ex.scheduleWithFixedDelay(new UploadLogTask(upload), 1000, 10000, TimeUnit.MILLISECONDS);
      upload.addProgressListener(new ProgressListener()
      {
        @Override
        public void progressChanged(ProgressEvent progressEvent)
        {
          if (progressEvent.getEventType() == ProgressEventType.TRANSFER_COMPLETED_EVENT)
          {
            log.debug("HELPER: Upload [{}] completed on [{}]", tmpFile.getName(), bucketName);
            ex.shutdown();
          }
        }
      });
      upload.waitForCompletion();
    }
    catch (AmazonClientException | InterruptedException e)
    {
      log.error("HELPER: Error Occurs:", e);
      tm.abortMultipartUploads(bucketName, new Date());
      ex.shutdown();
    }
  }

  class UploadLogTask implements Runnable
  {
    private Upload upload;

    public UploadLogTask(Upload u)
    {
      upload = u;
    }

    @Override
    public void run()
    {
      log.debug("HELPER: {}% uploaded.", upload.getProgress().getPercentTransferred());
    }
  }

  class DownloadLogTask implements Runnable
  {
    private Download download;

    public DownloadLogTask(Download d)
    {
      download = d;
    }

    @Override
    public void run()
    {
      log.debug("HELPER: {}% downloaed.", download.getProgress().getPercentTransferred());
    }
  }

  class CopyLogTask implements Runnable
  {
    private Copy copy;

    public CopyLogTask(Copy c)
    {
      copy = c;
    }

    @Override
    public void run()
    {
      log.debug("HELPER: {}% copied.", copy.getProgress().getPercentTransferred());
    }
  }
}
