package com.kddi.kos.examples.copy.multipart;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AbortMultipartUploadRequest;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.CompleteMultipartUploadRequest;
import com.amazonaws.services.s3.model.CompleteMultipartUploadResult;
import com.amazonaws.services.s3.model.CopyPartRequest;
import com.amazonaws.services.s3.model.CopyPartResult;
import com.amazonaws.services.s3.model.GetObjectMetadataRequest;
import com.amazonaws.services.s3.model.InitiateMultipartUploadRequest;
import com.amazonaws.services.s3.model.InitiateMultipartUploadResult;
import com.amazonaws.services.s3.model.ObjectMetadata;
import com.amazonaws.services.s3.model.PartETag;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトのコピー
 * <li>マルチパートアップロード API を使用したオブジェクトのコピー
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.copy.multipart.LowLevel_LargeObjectCopy
 * </pre>
 * 
 * @author ito
 * @deprecated 2015-06-25 cleversafe
 *             v3.5.1はCopyPartResultでlastModifiedを返さない事からaws-sdk-java-
 *             s3がエラー終了してしまう問題が解決されていない。
 */
public class LowLevel_LargeObjectCopy implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(LowLevel_LargeObjectCopy.class);

  public static void main(String[] args) throws IOException
  {
    new LowLevel_LargeObjectCopy().exec();
  }

  // Helper function that constructs ETags.
  static List<PartETag> GetETags(List<CopyPartResult> responses)
  {
    List<PartETag> etags = new ArrayList<PartETag>();
    for (CopyPartResult response : responses)
    {
      etags.add(new PartETag(response.getPartNumber(), response.getETag()));
    }
    return etags;
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String sourceBucketName = ClientProvider.getInstance().bucketName1;
    String targetBucketName = ClientProvider.getInstance().bucketName2;
    String baseObjectKey = "LowLevel_LargeObjectCopy";
    String sourceObjectKey = baseObjectKey + "_src.txt";
    String targetObjectKey = baseObjectKey + "_dest.txt";
    InitiateMultipartUploadResult initResult = null;// 異常発生時の中断処理のため

    try
    {
      // サンプル外操作：本サンプルで使ったkeyのみを対象にオブジェクト及びバージョン削除を試行
      new UtilProvider(s3Client).deleteObjectsVersions(sourceBucketName, baseObjectKey);
      new UtilProvider(s3Client).deleteObjectsVersions(targetBucketName, baseObjectKey);
      // サンプル外操作：10MBのファイルをtmpに作成後、指定bucketにアップロードする
      new MockCreator(s3Client).createTextFile(sourceObjectKey, 5L * MockCreator.SIZE_GB).uploadTo(sourceBucketName);
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在する事を確認する
      new UtilProvider(s3Client).showObjectsVersions(sourceBucketName, sourceObjectKey);

      // 2. 低レベルAPIを初期化する
      InitiateMultipartUploadRequest initiateRequest = new InitiateMultipartUploadRequest(targetBucketName, targetObjectKey);
      initResult = s3Client.initiateMultipartUpload(initiateRequest);
      log.info("upload initalized: upload_id[{}]", initResult.getUploadId());
      List<CopyPartResult> copyResponses = new ArrayList<CopyPartResult>();

      // 3. コピー元オブジェクトの詳細を取得する
      GetObjectMetadataRequest metadataRequest = new GetObjectMetadataRequest(sourceBucketName, sourceObjectKey);
      ObjectMetadata metadataResult = s3Client.getObjectMetadata(metadataRequest);
      long contentLength = metadataResult.getContentLength(); // in bytes
      long partSize = 5 * (long) Math.pow(2.0, 20.0); // 5 MB
      log.info("Content Length: {}", contentLength);
      log.info("Part Size: {}", partSize);

      // 4. パートアップロードを実行（シーケンシャル）
      long bytePosition = 0;
      for (int i = 1; bytePosition < contentLength; i++)
      {
        // 4-1. 最後のパートは5MBより小さいので調整する？
        partSize = Math.min(partSize, (contentLength - bytePosition));

        // 4-2. コピーパートリクエストを作成
        CopyPartRequest copyRequest =
                new CopyPartRequest().withDestinationBucketName(targetBucketName)
                                     .withDestinationKey(targetObjectKey)
                                     .withSourceBucketName(sourceBucketName)
                                     .withSourceKey(sourceObjectKey)
                                     .withUploadId(initResult.getUploadId())
                                     .withFirstByte(bytePosition)
                                     .withLastByte(bytePosition + partSize - 1 >= contentLength ? contentLength - 1 : bytePosition + partSize - 1)
                                     .withPartNumber(i);

        // 4-3. コピーパートとレスポンスをリストへ追加
        CopyPartResult result = s3Client.copyPart(copyRequest); // CopyPartResultオブジェクトが作成出来ないサービス的な問題
        copyResponses.add(result);

        bytePosition += partSize;
      }

      // 5. コピー完了
      CompleteMultipartUploadRequest completeRequest =
              new CompleteMultipartUploadRequest(targetBucketName, targetObjectKey, initResult.getUploadId(), GetETags(copyResponses));

      CompleteMultipartUploadResult result = s3Client.completeMultipartUpload(completeRequest);
      log.info("multipart upload completed: {}:{} - [{}]", result.getKey(), result.getVersionId(), result.getETag());

      // サンプル外操作：ObjectStorageにアップロードできている事を確認する
      new UtilProvider(s3Client).showObjectsVersions(targetBucketName, targetObjectKey);
    }
    catch (AmazonS3Exception e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(targetBucketName, targetObjectKey, uploadId);
      s3Client.abortMultipartUpload(request);

      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(targetBucketName, targetObjectKey, uploadId);
      s3Client.abortMultipartUpload(request);

      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(targetBucketName, targetObjectKey, uploadId);
      s3Client.abortMultipartUpload(request);

      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (FileNotFoundException e)
    {
      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
