package com.kddi.kos.examples.copy.single;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.CopyObjectRequest;
import com.amazonaws.services.s3.model.CopyObjectResult;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトのコピー
 * <li>1 回のオペレーションでのオブジェクトのコピー
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.copy.single.CopyObjectSingleOperation
 * </pre>
 * 
 * @author ito
 *
 */
public class CopyObjectSingleOperation implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(CopyObjectSingleOperation.class);

  public static void main(String[] args) throws IOException
  {
    new CopyObjectSingleOperation().exec();
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String sourceBucketName = ClientProvider.getInstance().bucketName1;
    String targetBucketName = ClientProvider.getInstance().bucketName2;
    String baseObjectKey = "CopyObjectSingleOperation";
    String sourceObjectKey = baseObjectKey + "_src.txt";
    String targetObjectKey = baseObjectKey + "_dest.txt";

    try
    {
      // サンプル外操作：本サンプルで使ったkeyのみを対象にオブジェクト及びバージョン削除を試行
      new UtilProvider(s3Client).deleteObjectsVersions(sourceBucketName, sourceObjectKey);
      new UtilProvider(s3Client).deleteObjectsVersions(targetBucketName, targetObjectKey);
      // サンプル外操作：10MBのファイルをtmpに作成後、指定Bucketにアップロード
      long mock_size = 10L * MockCreator.SIZE_MB;
      new MockCreator(s3Client).createTextFile(sourceObjectKey, mock_size).uploadTo(sourceBucketName);
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在する事を確認する
      new UtilProvider(s3Client).showObjectsVersions(sourceBucketName, sourceObjectKey);

      // 2. コピー元のオブジェクトをコピー先オブジェクトにコピーする
      CopyObjectRequest copyObjRequest = new CopyObjectRequest(sourceBucketName, sourceObjectKey, targetBucketName, targetObjectKey);
      CopyObjectResult copyObjectResult = s3Client.copyObject(copyObjRequest);
      log.info("Object copied. object {} - {}", copyObjRequest.getDestinationKey(), copyObjectResult.getETag());

      // サンプル外操作：ObjectStorageにアップロードできている事を確認する
      new UtilProvider(s3Client).showObjectsVersions(targetBucketName, targetObjectKey);
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (FileNotFoundException e)
    {
      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
