package com.kddi.kos.examples.delete.multi;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.DeleteObjectsRequest;
import com.amazonaws.services.s3.model.DeleteObjectsRequest.KeyVersion;
import com.amazonaws.services.s3.model.DeleteObjectsResult;
import com.amazonaws.services.s3.model.DeleteObjectsResult.DeletedObject;
import com.amazonaws.services.s3.model.ListObjectsRequest;
import com.amazonaws.services.s3.model.ObjectListing;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトの削除
 * <li>1 件のリクエストで複数オブジェクトを削除
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.delete.multi.DeleteMultipleObjects
 * </pre>
 * 
 * @author ito
 *
 */
public class DeleteMultipleObjects implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(DeleteMultipleObjects.class);

  public static void main(String[] args)
  {
    new DeleteMultipleObjects().exec();
  }

  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String keyName = "DeleteMultipleObjects.txt";
    String bucketName = ClientProvider.getInstance().bucketName1;

    try
    {
      // サンプル外操作：1MBのファイルをtmpに作成後、指定bucketにアップロードする
      new MockCreator(s3Client).createTextFile(keyName, 1L * MockCreator.SIZE_MB).duplicate(bucketName, 50);
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在する事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);

      // 2. オブジェクトのリストを収集する（前方一致のkey名のみ）
      ListObjectsRequest listObjectsRequest = new ListObjectsRequest().withBucketName(bucketName).withPrefix(keyName);
      ObjectListing objectListing;
      do
      {
        // 2-1.最初の1000件を取得
        List<KeyVersion> keys = new ArrayList<KeyVersion>();
        objectListing = s3Client.listObjects(listObjectsRequest);

        // 2-2. オブジェクトのリストを設定してまとめて削除をする
        for (S3ObjectSummary objectSummary : objectListing.getObjectSummaries())
        {
          // 注意：メモリ管理
          keys.add(new KeyVersion(objectSummary.getKey()));
        }
        DeleteObjectsRequest deleteObjectsRequest = new DeleteObjectsRequest(bucketName);
        deleteObjectsRequest.setKeys(keys);
        DeleteObjectsResult result = s3Client.deleteObjects(deleteObjectsRequest);
        for (DeletedObject deleted : result.getDeletedObjects())
        {
          log.info("delete successfully: {}", deleted.getKey());
        }

        // 2-3. 次の1000件を取得する準備をする
        listObjectsRequest.setMarker(objectListing.getNextMarker());
      }
      while (objectListing.isTruncated());
      // 3. 全てリストを取得し終えたら終了

      // サンプル外操作：ObjectStorageに存在しない事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (FileNotFoundException e)
    {
      log.error("Error Occurs: {}", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
