package com.kddi.kos.examples.upload.multipart;

import java.io.File;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.transfer.TransferManager;
import com.amazonaws.services.s3.transfer.Upload;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトのアップロード
 * <li>高レベル Java API を使用したマルチパートアップロード
 * <li>ファイルのアップロード
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.upload.multipart.UploadObjectMultipartUploadUsingHighLevelAPI
 * </pre>
 * 
 * @author ito
 *
 */
public class UploadObjectMultipartUploadUsingHighLevelAPI implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(UploadObjectMultipartUploadUsingHighLevelAPI.class);

  public static void main(String[] args) throws Exception
  {
    new UploadObjectMultipartUploadUsingHighLevelAPI().exec();
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String bucketName = ClientProvider.getInstance().bucketName1;
    String keyName = "UploadObjectMultipartUploadUsingHighLevelAPI.txt";

    try
    {
      // サンプル外操作：本サンプルで使ったkeyのみを対象にオブジェクト及びバージョン削除を試行
      new UtilProvider(s3Client).deleteObjectsVersions(bucketName, keyName);
      // サンプル外操作：1MBのファイルをtmpに作成
      String filePath = new MockCreator(s3Client).createTextFile(keyName, 1L * MockCreator.SIZE_MB).filePath;
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在しない事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);

      // 2. 高レベルAPIの準備
      TransferManager tm = new TransferManager(s3Client);
      Upload upload = tm.upload(bucketName, keyName, new File(filePath));

      // 3. 高レベルAPIでアップロードが完了するまで待機する
      upload.waitForCompletion();
      log.info("Upload {}[{}] completed.", keyName, filePath);

      // サンプル外操作：ObjectStorageにアップロードできている事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);
    }
    catch (AmazonS3Exception e)
    {
      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (InterruptedException e)
    {
      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
