package com.kddi.kos.examples.upload.multipart;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.AbortMultipartUploadRequest;
import com.amazonaws.services.s3.model.AmazonS3Exception;
import com.amazonaws.services.s3.model.CompleteMultipartUploadRequest;
import com.amazonaws.services.s3.model.CompleteMultipartUploadResult;
import com.amazonaws.services.s3.model.InitiateMultipartUploadRequest;
import com.amazonaws.services.s3.model.InitiateMultipartUploadResult;
import com.amazonaws.services.s3.model.PartETag;
import com.amazonaws.services.s3.model.UploadPartRequest;
import com.amazonaws.services.s3.model.UploadPartResult;
import com.kddi.helpers.creator.MockCreator;
import com.kddi.helpers.provider.ClientProvider;
import com.kddi.helpers.provider.UtilProvider;
import com.kddi.kos.examples.IExample;

/**
 * <ol>
 * <li>オブジェクトのアップロード
 * <li>低レベル Java API を使用したマルチパートアップロード
 * <li>ファイルのアップロード
 * </ol>
 * 
 * <pre>
 * mvn exec:java -Dexec.mainClass=com.kddi.kos.examples.upload.multipart.UploadObjectMultipartUploadUsingLowLevelAPI
 * </pre>
 * 
 * @author ito
 *
 */
public class UploadObjectMultipartUploadUsingLowLevelAPI implements IExample
{
  private static final Logger log = LoggerFactory.getLogger(UploadObjectMultipartUploadUsingLowLevelAPI.class);

  public static void main(String[] args) throws IOException
  {
    new UploadObjectMultipartUploadUsingLowLevelAPI().exec();
  }

  @Override
  public void exec()
  {
    // 1. clientの準備
    AmazonS3 s3Client = ClientProvider.getInstance().getS3Client();
    String bucketName = ClientProvider.getInstance().bucketName1;
    String keyName = "UploadObjectMultipartUploadUsingLowLevelAPI.txt";
    InitiateMultipartUploadResult initResult = null;// 異常発生時の中断処理のため

    try
    {
      // サンプル外操作：本サンプルで使ったkeyのみを対象にオブジェクト及びバージョン削除を試行
      new UtilProvider(s3Client).deleteObjectsVersions(bucketName, keyName);
      // サンプル外操作：10MBのファイルをtmpに作成
      String filePath = new MockCreator(s3Client).createTextFile(keyName, 10L * MockCreator.SIZE_MB).filePath;
      // サンプル外操作：本サンプルで使うkeyがstorage上に存在しない事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);

      // 2. 低レベルAPIを初期化する
      InitiateMultipartUploadRequest initRequest = new InitiateMultipartUploadRequest(bucketName, keyName);
      initResult = s3Client.initiateMultipartUpload(initRequest);
      log.info("upload initalized: upload_id[{}]", initResult.getUploadId());

      // 3. アップロードするファイルの詳細を決める
      File file = new File(filePath);
      long contentLength = file.length();
      long partSize = 5L * MockCreator.SIZE_MB;
      log.info("Content Length: {}", contentLength);
      log.info("Part Size: {}", partSize);

      // 4. パートアップロードを実行（シーケンシャル）
      List<PartETag> partETags = new ArrayList<PartETag>();
      long filePosition = 0;
      for (int i = 1; filePosition < contentLength; i++)
      {
        // 4-1. 最後のパートは5MBより小さいので調整する
        partSize = Math.min(partSize, (contentLength - filePosition));

        // 4-2. アップロードパートリクエストを作成
        UploadPartRequest uploadRequest =
                new UploadPartRequest().withBucketName(bucketName)
                                       .withKey(keyName)
                                       .withUploadId(initResult.getUploadId())
                                       .withPartNumber(i)
                                       .withFileOffset(filePosition)
                                       .withFile(file)
                                       .withPartSize(partSize);

        // 4-3. アップロードパートとレスポンスをリストへ追加
        UploadPartResult result = s3Client.uploadPart(uploadRequest);
        log.info("part uploaded: {}({}) - {}", uploadRequest.getKey(), result.getPartNumber(), result.getPartETag()
                                                                                                     .getETag());
        partETags.add(result.getPartETag());

        filePosition += partSize;
      }

      // 5. アップロード完了
      String uploadId = initResult.getUploadId();
      CompleteMultipartUploadRequest compRequest =
              new CompleteMultipartUploadRequest(bucketName, keyName, uploadId, partETags);
      CompleteMultipartUploadResult result = s3Client.completeMultipartUpload(compRequest);
      log.info("multipart upload completed: {}:{} - [{}]", result.getKey(), result.getVersionId(), result.getETag());

      // サンプル外操作：ObjectStorageにアップロードできている事を確認する
      new UtilProvider(s3Client).showObjectsVersions(bucketName, keyName, true);
    }
    catch (AmazonS3Exception e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(bucketName, keyName, uploadId);
      s3Client.abortMultipartUpload(request);

      // サービス側のリクエスト処理中エラー
      // リクエストは到達し、処理開始したが、途中でエラーが発生したケース
      // アクセスキー間違いなど
      log.error("Error Occurs:", e);
    }
    catch (AmazonServiceException e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(bucketName, keyName, uploadId);
      s3Client.abortMultipartUpload(request);

      // サービス側のリクエスト処理開始時エラー
      // リクエストは到達したが処理開始出来なかったケース
      log.error("Error Occurs:", e);
    }
    catch (AmazonClientException e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(bucketName, keyName, uploadId);
      s3Client.abortMultipartUpload(request);

      // クライアント側のレスポンス処理開始エラー
      // サービス側からのレスポンスが得られない場合や、レスポンスが解釈出来なかったケース
      // copyPart()を実行するなど
      log.error("Error Occurs:", e);
    }
    catch (Exception e)
    {
      // x. 異常時はアップロードを中断する
      String uploadId = initResult.getUploadId();
      AbortMultipartUploadRequest request = new AbortMultipartUploadRequest(bucketName, keyName, uploadId);
      s3Client.abortMultipartUpload(request);

      log.error("Error Occurs:", e);
    }
    finally
    {
      log.info("END");
    }
  }
}
